@@ -4,19 +4,25 @@
 
 module Text.XmlHtml.Common where
 
+import           Data.ByteString (ByteString)
 import           Blaze.ByteString.Builder
+import qualified Data.ByteString.Char8 as BS
+import qualified Data.ByteString.Builder as B
 import           Data.Char (isAscii, isLatin1)
 import qualified Data.HashMap.Strict as M
+import qualified Data.HashSet as S
+import qualified Data.Map.Strict as Map
 import           Data.Maybe
 
 import           Data.Text (Text)
 import qualified Data.Text as T
 import qualified Data.Text.Encoding as T
 import qualified Data.Text.Encoding.Error as TE
+import qualified Data.Text.Lazy as TL
+import qualified Data.Text.Lazy.Encoding as TL
 
-import           Data.ByteString (ByteString)
-import qualified Data.ByteString.Char8 as BS
-import           Text.XmlHtml.HTML.Meta (reversePredefinedRefs)
+import           Text.XmlHtml.HTML.Meta (reversePredefinedRefs,
+                                         explicitAttributes)
 
 
 ------------------------------------------------------------------------------
@@ -50,6 +56,37 @@ data Node = TextNode !Text
     deriving (Eq, Show)
 
 
+------------------------------------------------------------------------------
+-- | Rendering options
+data RenderOptions = RenderOptions {
+      roAttributeSurround :: AttrSurround
+      -- ^ Single or double-quotes used around attribute values
+
+    , roAttributeResolveInternal :: AttrResolveInternalQuotes
+      -- ^ Quotes inside attribute values that conflict with the surround
+      -- are escaped, or the outer quotes are changed to avoid conflicting
+      -- with the internal ones
+
+    , roExplicitEmptyAttrs :: Maybe (M.HashMap Text (S.HashSet Text))
+      -- ^ Attributes in the whitelist with empty values are
+      -- rendered as <div example="">
+      -- 'Nothing' applies this rule to all attributes with empty values
+
+    } deriving (Eq, Show)
+
+data AttrSurround = SurroundDoubleQuote | SurroundSingleQuote
+    deriving (Eq, Ord, Show)
+
+data AttrResolveInternalQuotes = AttrResolveByEscape | AttrResolveAvoidEscape
+    deriving (Eq, Ord, Show)
+
+defaultRenderOptions :: RenderOptions
+defaultRenderOptions = RenderOptions
+    { roAttributeSurround        = SurroundSingleQuote
+    , roAttributeResolveInternal = AttrResolveAvoidEscape
+    , roExplicitEmptyAttrs       = Just explicitAttributes
+    }
+
 ------------------------------------------------------------------------------
 -- | Determines whether the node is text or not.
 isTextNode :: Node -> Bool
@@ -236,7 +273,7 @@ encodeIso_8859_1 t = T.encodeUtf8 . T.concat . map toAsciiChunk $
     -- Otherwise its unicode index is printed to decimal and "&#" is appended
     toAsciiChar c = maybe
         (uniEscape c) (\esc -> T.concat ["&", esc, ";"])
-        (M.lookup (T.singleton c) reversePredefinedRefs)
+        (Map.lookup (T.singleton c) reversePredefinedRefs)
 
     uniEscape = T.append "&#" . flip T.snoc ';' . T.pack .
                 (show :: Int -> String) . fromEnum
@@ -261,3 +298,11 @@ isUTF16 e = e == UTF16BE || e == UTF16LE
 fromText :: Encoding -> Text -> Builder
 fromText e t = fromByteString (encoder e t)
 
+
+bmap :: (Text -> Text) -> B.Builder -> B.Builder
+bmap f   = B.byteString
+               . T.encodeUtf8
+               . f
+               . TL.toStrict
+               . TL.decodeUtf8
+               . B.toLazyByteString
