@@ -17,6 +17,7 @@ import qualified Text.Parsec as P
 
 import qualified Data.HashSet as S
 import qualified Data.HashMap.Strict as M
+import qualified Data.Map.Strict as Map
 
 import           Data.Text (Text)
 import qualified Data.Text as T
@@ -198,7 +199,7 @@ emptyOrStartTag = do
 ------------------------------------------------------------------------------
 attrName :: Parser Text
 attrName = takeWhile1 isAttrName
-  where isAttrName c | c `elem` "\0 \"\'>/=" = False
+  where isAttrName c | c `elem` ['\0',' ','"','\'','>','/','='] = False
                      | isControlChar c       = False
                      | otherwise             = True
 
@@ -216,15 +217,15 @@ isControlChar c | c >= '\x007F' && c <= '\x009F' = True
 quotedAttrValue :: Parser Text
 quotedAttrValue = singleQuoted <|> doubleQuoted
   where
-    singleQuoted = P.char '\'' *> refTill "&\'" <* P.char '\''
-    doubleQuoted = P.char '\"' *> refTill "&\"" <* P.char '\"'
+    singleQuoted = P.char '\'' *> refTill ['&','\''] <* P.char '\''
+    doubleQuoted = P.char '"'  *> refTill ['&','"']  <* P.char '"'
     refTill end = T.concat <$> many
         (takeWhile1 (not . (`elem` end)) <|> reference)
 
 
 ------------------------------------------------------------------------------
 unquotedAttrValue :: Parser Text
-unquotedAttrValue = refTill " \"\'=<>&`"
+unquotedAttrValue = refTill [' ','"','\'','=','<','>','&','`']
   where
     refTill end = T.concat <$> some
         (takeWhile1 (not . (`elem` end)) <|> reference)
@@ -331,7 +332,7 @@ reference = do
             when (not (isValidChar c)) $ fail $
                 "Reference is not a valid character"
             return (T.singleton c)
-        Right nm -> case M.lookup nm predefinedRefs of
+        Right nm -> case Map.lookup nm predefinedRefs of
             Nothing -> fail $ "Unknown entity reference: " ++ T.unpack nm
             Just t  -> return t
 
