@@ -1,78 +1,55 @@
 {-# LANGUAGE ExistentialQuantification #-}
 {-# LANGUAGE OverloadedStrings         #-}
 
-module Text.XmlHtml.Tests (tests) where
+module Text.XmlHtml.Tests where
 
 import           Blaze.ByteString.Builder
 import           Data.ByteString (ByteString)
 import qualified Data.ByteString.Char8 as B
-import           Data.Monoid
+import           Data.Monoid (mappend, mempty)
 import           Data.String
-import           Data.Text ()                  -- for string instance
+import           Data.Text (Text)
 import qualified Data.Text.Encoding as T
-import           Test.Framework
-import           Test.Framework.Providers.HUnit
+import           Test.Hspec
 import           Test.HUnit hiding (Test, Node)
 import           Text.Blaze
 import           Text.Blaze.Html
 import qualified Text.Blaze.Html5 as H
 import qualified Text.Blaze.Html5.Attributes as A
-import qualified Text.Blaze.Internal as H
 import           Text.Blaze.Renderer.XmlHtml
 import           Text.XmlHtml
-import           Text.XmlHtml.CursorTests
-import           Text.XmlHtml.DocumentTests
 import           Text.XmlHtml.TestCommon
-import           Text.XmlHtml.OASISTest
-
-
-------------------------------------------------------------------------------
--- The master list of tests to run. ------------------------------------------
-------------------------------------------------------------------------------
-
-tests :: [Test]
-tests = xmlParsingTests
-     ++ htmlXMLParsingTests
-     ++ htmlParsingQuirkTests
-     ++ xmlRenderingTests
-     ++ htmlXMLRenderingTests
-     ++ htmlRenderingQuirkTests
-     ++ documentTests
-     ++ cursorTests
-     ++ blazeRenderTests
-     ++ testsOASIS
 
 
 ------------------------------------------------------------------------------
 -- XML Parsing Tests ---------------------------------------------------------
 ------------------------------------------------------------------------------
 
-xmlParsingTests :: [Test]
-xmlParsingTests = [
-    testCase "byteOrderMark          " byteOrderMark,
-    testIt   "emptyDocument          " emptyDocument,
-    testIt   "publicDocType          " publicDocType,
-    testIt   "systemDocType          " systemDocType,
-    testIt   "emptyDocType           " emptyDocType,
-    testCase "dtdInternalScan        " dtdInternalScan,
-    testIt   "textOnly               " textOnly,
-    testIt   "textWithRefs           " textWithRefs,
-    testIt   "untermRef              " untermRef,
-    testIt   "textWithCDATA          " textWithCDATA,
-    testIt   "cdataOnly              " cdataOnly,
-    testIt   "commentOnly            " commentOnly,
-    testIt   "emptyElement           " emptyElement,
-    testIt   "emptyElement2          " emptyElement2,
-    testIt   "elemWithText           " elemWithText,
-    testIt   "xmlDeclXML             " xmlDeclXML,
-    testIt   "procInst               " procInst,
-    testIt   "badDoctype1            " badDoctype1,
-    testIt   "badDoctype2            " badDoctype2,
-    testIt   "badDoctype3            " badDoctype3,
-    testIt   "badDoctype4            " badDoctype4,
-    testIt   "badDoctype5            " badDoctype5,
-    testCase "tagNames               " tagNames
-    ]
+xmlParsingTests :: Spec
+xmlParsingTests = do
+    it       "byteOrderMark          " byteOrderMark
+    testIt   "emptyDocument          " emptyDocument
+    testIt   "publicDocType          " publicDocType
+    testIt   "systemDocType          " systemDocType
+    testIt   "emptyDocType           " emptyDocType
+    it       "dtdInternalScan        " dtdInternalScan
+    testIt   "textOnly               " textOnly
+    testIt   "textWithRefs           " textWithRefs
+    testIt   "untermRef              " untermRef
+    testIt   "textWithCDATA          " textWithCDATA
+    testIt   "cdataOnly              " cdataOnly
+    testIt   "commentOnly            " commentOnly
+    testIt   "emptyElement           " emptyElement
+    testIt   "emptyElement2          " emptyElement2
+    testIt   "elemWithText           " elemWithText
+    testIt   "xmlDeclXML             " xmlDeclXML
+    testIt   "procInst               " procInst
+    testIt   "badDoctype1            " badDoctype1
+    testIt   "badDoctype2            " badDoctype2
+    testIt   "badDoctype3            " badDoctype3
+    testIt   "badDoctype4            " badDoctype4
+    testIt   "badDoctype5            " badDoctype5
+    it       "tagNames               " tagNames
 
 byteOrderMark :: Assertion
 byteOrderMark = do
@@ -253,28 +230,27 @@ tagNames = do
 -- HTML Repetitions of XML Parsing Tests -------------------------------------
 ------------------------------------------------------------------------------
 
-htmlXMLParsingTests :: [Test]
-htmlXMLParsingTests = [
-    testIt "emptyDocumentHTML      " emptyDocumentHTML,
-    testIt "publicDocTypeHTML      " publicDocTypeHTML,
-    testIt "systemDocTypeHTML      " systemDocTypeHTML,
-    testIt "emptyDocTypeHTML       " emptyDocTypeHTML,
-    testIt "textOnlyHTML           " textOnlyHTML,
-    testIt "textWithRefsHTML       " textWithRefsHTML,
-    testIt "textWithCDataHTML      " textWithCDataHTML,
-    testIt "cdataOnlyHTML          " cdataOnlyHTML,
-    testIt "commentOnlyHTML        " commentOnlyHTML,
-    testIt "emptyElementHTML       " emptyElementHTML,
-    testIt "emptyElement2HTML      " emptyElement2HTML,
-    testIt "elemWithTextHTML       " elemWithTextHTML,
-    testIt "xmlDeclHTML            " xmlDeclHTML,
-    testIt "procInstHTML           " procInstHTML,
-    testIt "badDoctype1HTML        " badDoctype1HTML,
-    testIt "badDoctype2HTML        " badDoctype2HTML,
-    testIt "badDoctype3HTML        " badDoctype3HTML,
-    testIt "badDoctype4HTML        " badDoctype4HTML,
+htmlXMLParsingTests :: Spec
+htmlXMLParsingTests = do
+    testIt "emptyDocumentHTML      " emptyDocumentHTML
+    testIt "publicDocTypeHTML      " publicDocTypeHTML
+    testIt "systemDocTypeHTML      " systemDocTypeHTML
+    testIt "emptyDocTypeHTML       " emptyDocTypeHTML
+    testIt "textOnlyHTML           " textOnlyHTML
+    testIt "textWithRefsHTML       " textWithRefsHTML
+    testIt "textWithCDataHTML      " textWithCDataHTML
+    testIt "cdataOnlyHTML          " cdataOnlyHTML
+    testIt "commentOnlyHTML        " commentOnlyHTML
+    testIt "emptyElementHTML       " emptyElementHTML
+    testIt "emptyElement2HTML      " emptyElement2HTML
+    testIt "elemWithTextHTML       " elemWithTextHTML
+    testIt "xmlDeclHTML            " xmlDeclHTML
+    testIt "procInstHTML           " procInstHTML
+    testIt "badDoctype1HTML        " badDoctype1HTML
+    testIt "badDoctype2HTML        " badDoctype2HTML
+    testIt "badDoctype3HTML        " badDoctype3HTML
+    testIt "badDoctype4HTML        " badDoctype4HTML
     testIt "badDoctype5HTML        " badDoctype5HTML
-    ]
 
 emptyDocumentHTML :: Bool
 emptyDocumentHTML = parseHTML "" ""
@@ -353,43 +329,42 @@ badDoctype5HTML    = isLeft $ parseHTML "" ("<!DOCTYPE html SYSTEM \"foo\" "
 -- HTML Quirks Parsing Tests -------------------------------------------------
 ------------------------------------------------------------------------------
 
-htmlParsingQuirkTests :: [Test]
-htmlParsingQuirkTests = [
-    testIt   "voidElem               " voidElem,
-    testIt   "caseInsDoctype1        " caseInsDoctype1,
-    testIt   "caseInsDoctype2        " caseInsDoctype2,
-    testIt   "voidEmptyElem          " voidEmptyElem,
-    testIt   "rawTextElem            " rawTextElem,
-    testIt   "endTagCase             " endTagCase,
-    testIt   "hexEntityCap           " hexEntityCap,
-    testIt   "laxAttrName            " laxAttrName,
-    testCase "badAttrName            " badAttrName,
-    testIt   "emptyAttr              " emptyAttr,
-    testIt   "emptyAttr2             " emptyAttr2,
-    testIt   "unquotedAttr           " unquotedAttr,
-    testIt   "laxAttrVal             " laxAttrVal,
-    testIt   "ampersandInText        " ampersandInText,
-    testIt   "omitOptionalEnds       " omitOptionalEnds,
-    testIt   "omitEndHEAD            " omitEndHEAD,
-    testIt   "omitEndLI              " omitEndLI,
-    testIt   "omitEndDT              " omitEndDT,
-    testIt   "omitEndDD              " omitEndDD,
-    testIt   "omitEndP               " omitEndP,
-    testIt   "omitEndRT              " omitEndRT,
-    testIt   "omitEndRP              " omitEndRP,
-    testIt   "omitEndOPTGRP          " omitEndOPTGRP,
-    testIt   "omitEndOPTION          " omitEndOPTION,
-    testIt   "omitEndCOLGRP          " omitEndCOLGRP,
-    testIt   "omitEndTHEAD           " omitEndTHEAD,
-    testIt   "omitEndTBODY           " omitEndTBODY,
-    testIt   "omitEndTFOOT           " omitEndTFOOT,
-    testIt   "omitEndTR              " omitEndTR,
-    testIt   "omitEndTD              " omitEndTD,
-    testIt   "omitEndTH              " omitEndTH,
-    testIt   "testNewRefs            " testNewRefs,
-    testIt   "errorImplicitClose     " errorImplicitClose,
+htmlParsingQuirkTests :: Spec
+htmlParsingQuirkTests = do
+    testIt   "voidElem               " voidElem
+    testIt   "caseInsDoctype1        " caseInsDoctype1
+    testIt   "caseInsDoctype2        " caseInsDoctype2
+    testIt   "voidEmptyElem          " voidEmptyElem
+    testIt   "rawTextElem            " rawTextElem
+    testIt   "endTagCase             " endTagCase
+    testIt   "hexEntityCap           " hexEntityCap
+    testIt   "laxAttrName            " laxAttrName
+    it       "badAttrName            " badAttrName
+    testIt   "emptyAttr              " emptyAttr
+    testIt   "emptyAttr2             " emptyAttr2
+    testIt   "unquotedAttr           " unquotedAttr
+    testIt   "laxAttrVal             " laxAttrVal
+    testIt   "ampersandInText        " ampersandInText
+    testIt   "omitOptionalEnds       " omitOptionalEnds
+    testIt   "omitEndHEAD            " omitEndHEAD
+    testIt   "omitEndLI              " omitEndLI
+    testIt   "omitEndDT              " omitEndDT
+    testIt   "omitEndDD              " omitEndDD
+    testIt   "omitEndP               " omitEndP
+    testIt   "omitEndRT              " omitEndRT
+    testIt   "omitEndRP              " omitEndRP
+    testIt   "omitEndOPTGRP          " omitEndOPTGRP
+    testIt   "omitEndOPTION          " omitEndOPTION
+    testIt   "omitEndCOLGRP          " omitEndCOLGRP
+    testIt   "omitEndTHEAD           " omitEndTHEAD
+    testIt   "omitEndTBODY           " omitEndTBODY
+    testIt   "omitEndTFOOT           " omitEndTFOOT
+    testIt   "omitEndTR              " omitEndTR
+    testIt   "omitEndTD              " omitEndTD
+    testIt   "omitEndTH              " omitEndTH
+    testIt   "testNewRefs            " testNewRefs
+    testIt   "errorImplicitClose     " errorImplicitClose
     testIt   "weirdScriptThing       " weirdScriptThing
-    ]
 
 caseInsDoctype1 :: Bool
 caseInsDoctype1 = parseHTML "" "<!dOcTyPe html SyStEm 'foo'>"
@@ -548,24 +523,23 @@ weirdScriptThing = parseHTML "" "<div><script type=\"text/javascript\">selector.
 -- XML Rendering Tests -------------------------------------------------------
 ------------------------------------------------------------------------------
 
-xmlRenderingTests :: [Test]
-xmlRenderingTests = [
-    testIt "renderByteOrderMark    " renderByteOrderMark,
-    testIt "renderByteOrderMarkLE  " renderByteOrderMarkLE,
-    testIt "singleQuoteInSysID     " singleQuoteInSysID,
-    testIt "doubleQuoteInSysID     " doubleQuoteInSysID,
-    testIt "bothQuotesInSysID      " bothQuotesInSysID,
-    testIt "doubleQuoteInPubID     " doubleQuoteInPubID,
-    testIt "doubleDashInComment    " doubleDashInComment,
-    testIt "trailingDashInComment  " trailingDashInComment,
-    testIt "renderEmptyText        " renderEmptyText,
-    testIt "singleQuoteInAttr      " singleQuoteInAttr,
-    testIt "doubleQuoteInAttr      " doubleQuoteInAttr,
-    testIt "bothQuotesInAttr       " bothQuotesInAttr,
-    testIt "ndashEscapesInLatin    " ndashEscapesInLatin,
-    testIt "smileyEscapesInLatin   " smileyEscapesInLatin,
-    testIt "numericalEscapes       " numericalEscapes
-    ]
+xmlRenderingTests :: Spec
+xmlRenderingTests = do
+    testIt "renderByteOrderMark    " renderByteOrderMark
+    testIt "renderByteOrderMarkLE  " renderByteOrderMarkLE
+    testIt "singleQuoteInSysID     " singleQuoteInSysID
+    testIt "doubleQuoteInSysID     " doubleQuoteInSysID
+    testIt "bothQuotesInSysID      " bothQuotesInSysID
+    testIt "doubleQuoteInPubID     " doubleQuoteInPubID
+    testIt "doubleDashInComment    " doubleDashInComment
+    testIt "trailingDashInComment  " trailingDashInComment
+    testIt "renderEmptyText        " renderEmptyText
+    testIt "singleQuoteInAttr      " singleQuoteInAttr
+    testIt "doubleQuoteInAttr      " doubleQuoteInAttr
+    testIt "bothQuotesInAttr       " bothQuotesInAttr
+    testIt "ndashEscapesInLatin    " ndashEscapesInLatin
+    testIt "smileyEscapesInLatin   " smileyEscapesInLatin
+    numericalEscapes
 
 renderByteOrderMark :: Bool
 renderByteOrderMark =
@@ -584,21 +558,21 @@ utf8Decl = T.encodeUtf8 "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
 singleQuoteInSysID :: Bool
 singleQuoteInSysID =
     toByteString (render (XmlDocument UTF8
-        (Just (DocType "html" (System "test\'ing") NoInternalSubset))
+        (Just (DocType "html" (System "test'ing") NoInternalSubset))
         []))
-    == utf8Decl `B.append` "<!DOCTYPE html SYSTEM \"test\'ing\">\n"
+    == utf8Decl `B.append` "<!DOCTYPE html SYSTEM \"test'ing\">\n"
 
 doubleQuoteInSysID :: Bool
 doubleQuoteInSysID =
     toByteString (render (XmlDocument UTF8
         (Just (DocType "html" (System "test\"ing") NoInternalSubset))
         []))
-    == utf8Decl `B.append` "<!DOCTYPE html SYSTEM \'test\"ing\'>\n"
+    == utf8Decl `B.append` "<!DOCTYPE html SYSTEM 'test\"ing'>\n"
 
 bothQuotesInSysID :: Bool
 bothQuotesInSysID = isBottom $
     toByteString (render (XmlDocument UTF8
-        (Just (DocType "html" (System "test\"\'ing") NoInternalSubset))
+        (Just (DocType "html" (System "test\"'ing") NoInternalSubset))
         []))
 
 doubleQuoteInPubID :: Bool
@@ -629,23 +603,23 @@ renderEmptyText =
 singleQuoteInAttr :: Bool
 singleQuoteInAttr =
     toByteString (render (XmlDocument UTF8 Nothing [
-        Element "foo" [("bar", "test\'ing")] []
+        Element "foo" [("bar", "test'ing")] []
         ]))
-    == utf8Decl `B.append` "<foo bar=\"test\'ing\"/>"
+    == utf8Decl `B.append` "<foo bar=\"test'ing\"/>"
 
 doubleQuoteInAttr :: Bool
 doubleQuoteInAttr =
     toByteString (render (XmlDocument UTF8 Nothing [
         Element "foo" [("bar", "test\"ing")] []
         ]))
-    == utf8Decl `B.append` "<foo bar=\'test\"ing\'/>"
+    == utf8Decl `B.append` "<foo bar='test\"ing'/>"
 
 bothQuotesInAttr :: Bool
 bothQuotesInAttr =
     toByteString (render (XmlDocument UTF8 Nothing [
-        Element "foo" [("bar", "test\'\"ing")] []
+        Element "foo" [("bar", "test'\"ing")] []
         ]))
-    == utf8Decl `B.append` "<foo bar=\"test\'&quot;ing\"/>"
+    == utf8Decl `B.append` "<foo bar='test&apos;\"ing'/>"
 
 ndashEscapesInLatin :: Bool
 ndashEscapesInLatin =
@@ -661,31 +635,30 @@ smileyEscapesInLatin =
         ]))
     == "Hello &#9786;"
 
-numericalEscapes :: Bool
-numericalEscapes =
-    fmap (toByteString . renderXmlFragment ISO_8859_1 . docContent)
-    (parseXML "test" "Hello &#174;")
-    == Right "Hello &REG;"
+numericalEscapes :: Spec
+numericalEscapes = testEqual "numericalEscapes" (Right "Hello &reg;") actual
+  where
+    actual = toByteString . renderXmlFragment ISO_8859_1 . docContent
+               <$> parseXML "test" "Hello &#174;"
 
 
 ------------------------------------------------------------------------------
 -- HTML Repeats of XML Rendering Tests ---------------------------------------
 ------------------------------------------------------------------------------
 
-htmlXMLRenderingTests :: [Test]
-htmlXMLRenderingTests = [
-    testIt "hRenderByteOrderMark   " hRenderByteOrderMark,
-    testIt "hSingleQuoteInSysID    " hSingleQuoteInSysID,
-    testIt "hDoubleQuoteInSysID    " hDoubleQuoteInSysID,
-    testIt "hBothQuotesInSysID     " hBothQuotesInSysID,
-    testIt "hDoubleQuoteInPubID    " hDoubleQuoteInPubID,
-    testIt "hDoubleDashInComment   " hDoubleDashInComment,
-    testIt "hTrailingDashInComment " hTrailingDashInComment,
-    testIt "hRenderEmptyText       " hRenderEmptyText,
-    testIt "hSingleQuoteInAttr     " hSingleQuoteInAttr,
-    testIt "hDoubleQuoteInAttr     " hDoubleQuoteInAttr,
+htmlXMLRenderingTests :: Spec
+htmlXMLRenderingTests = do
+    testIt "hRenderByteOrderMark   " hRenderByteOrderMark
+    testIt "hSingleQuoteInSysID    " hSingleQuoteInSysID
+    testIt "hDoubleQuoteInSysID    " hDoubleQuoteInSysID
+    testIt "hBothQuotesInSysID     " hBothQuotesInSysID
+    testIt "hDoubleQuoteInPubID    " hDoubleQuoteInPubID
+    testIt "hDoubleDashInComment   " hDoubleDashInComment
+    testIt "hTrailingDashInComment " hTrailingDashInComment
+    testIt "hRenderEmptyText       " hRenderEmptyText
+    testIt "hSingleQuoteInAttr     " hSingleQuoteInAttr
+    testIt "hDoubleQuoteInAttr     " hDoubleQuoteInAttr
     testIt "hBothQuotesInAttr      " hBothQuotesInAttr
-    ]
 
 hRenderByteOrderMark :: Bool
 hRenderByteOrderMark =
@@ -695,21 +668,21 @@ hRenderByteOrderMark =
 hSingleQuoteInSysID :: Bool
 hSingleQuoteInSysID =
     toByteString (render (HtmlDocument UTF8
-        (Just (DocType "html" (System "test\'ing") NoInternalSubset))
+        (Just (DocType "html" (System "test'ing") NoInternalSubset))
         []))
-    == "<!DOCTYPE html SYSTEM \"test\'ing\">\n"
+    == "<!DOCTYPE html SYSTEM \"test'ing\">\n"
 
 hDoubleQuoteInSysID :: Bool
 hDoubleQuoteInSysID =
     toByteString (render (HtmlDocument UTF8
         (Just (DocType "html" (System "test\"ing") NoInternalSubset))
         []))
-    == "<!DOCTYPE html SYSTEM \'test\"ing\'>\n"
+    == "<!DOCTYPE html SYSTEM 'test\"ing'>\n"
 
 hBothQuotesInSysID :: Bool
 hBothQuotesInSysID = isBottom $
     toByteString (render (HtmlDocument UTF8
-        (Just (DocType "html" (System "test\"\'ing") NoInternalSubset))
+        (Just (DocType "html" (System "test\"'ing") NoInternalSubset))
         []))
 
 hDoubleQuoteInPubID :: Bool
@@ -740,58 +713,61 @@ hRenderEmptyText =
 hSingleQuoteInAttr :: Bool
 hSingleQuoteInAttr =
     toByteString (render (HtmlDocument UTF8 Nothing [
-        Element "foo" [("bar", "test\'ing")] []
+        Element "foo" [("bar", "test'ing")] []
         ]))
-    == "<foo bar=\"test\'ing\"></foo>"
+    == "<foo bar=\"test'ing\"></foo>"
 
 hDoubleQuoteInAttr :: Bool
 hDoubleQuoteInAttr =
     toByteString (render (HtmlDocument UTF8 Nothing [
         Element "foo" [("bar", "test\"ing")] []
         ]))
-    == "<foo bar=\'test\"ing\'></foo>"
+    == "<foo bar='test\"ing'></foo>"
 
 hBothQuotesInAttr :: Bool
 hBothQuotesInAttr =
     toByteString (render (HtmlDocument UTF8 Nothing [
-        Element "foo" [("bar", "test\'\"ing")] []
+        Element "foo" [("bar", "test'\"ing")] []
         ]))
-    == "<foo bar=\"test\'&quot;ing\"></foo>"
+    == "<foo bar='test&apos;\"ing'></foo>"
 
 
 ------------------------------------------------------------------------------
 -- HTML Quirks Rendering Tests -----------------------------------------------
 ------------------------------------------------------------------------------
 
-htmlRenderingQuirkTests :: [Test]
-htmlRenderingQuirkTests = [
-    testIt "renderHTMLVoid         " renderHTMLVoid,
-    testIt "renderHTMLVoid2        " renderHTMLVoid2,
-    testIt "renderHTMLRaw          " renderHTMLRaw,
-    testIt "renderHTMLRawMult      " renderHTMLRawMult,
-    testIt "renderHTMLRaw2         " renderHTMLRaw2,
-    testIt "renderHTMLRaw3         " renderHTMLRaw3,
-    testIt "renderHTMLRaw4         " renderHTMLRaw4,
-    testIt "renderHTMLEmptyAttr    " renderHTMLEmptyAttr,
-    testIt "renderHTMLEmptyAttr2   " renderHTMLEmptyAttr2,
-    testIt "renderHTMLAmpAttr1     " renderHTMLAmpAttr1,
-    testIt "renderHTMLAmpAttr2     " renderHTMLAmpAttr2,
-    testIt "renderHTMLAmpAttr3     " renderHTMLAmpAttr3,
-    testIt "renderHTMLQVoid        " renderHTMLQVoid,
-    testIt "renderHTMLQVoid2       " renderHTMLQVoid2,
-    testIt "renderHTMLQRaw         " renderHTMLQRaw,
-    testIt "renderHTMLQRawMult     " renderHTMLQRawMult,
-    testIt "renderHTMLQRaw2        " renderHTMLQRaw2,
-    testIt "renderHTMLQRaw3        " renderHTMLQRaw3,
+htmlRenderingQuirkTests :: Spec
+htmlRenderingQuirkTests = do
+    testIt "renderHTMLVoid         " renderHTMLVoid
+    testIt "renderHTMLVoid2        " renderHTMLVoid2
+    testIt "renderHTMLRaw          " renderHTMLRaw
+    testIt "renderHTMLRawMult      " renderHTMLRawMult
+    testIt "renderHTMLRaw2         " renderHTMLRaw2
+    testIt "renderHTMLRaw3         " renderHTMLRaw3
+    testIt "renderHTMLRaw4         " renderHTMLRaw4
+    testIt "renderHTMLEmptyAttr    " renderHTMLEmptyAttr
+    testIt "renderHTMLEmptyAttr2   " renderHTMLEmptyAttr2
+    testIt "renderHTMLAmpAttr1     " renderHTMLAmpAttr1
+    testIt "renderHTMLAmpAttr2     " renderHTMLAmpAttr2
+    testIt "renderHTMLAmpAttr3     " renderHTMLAmpAttr3
+    testIt "renderHTMLQVoid        " renderHTMLQVoid
+    testIt "renderHTMLQVoid2       " renderHTMLQVoid2
+    testIt "renderHTMLQRaw         " renderHTMLQRaw
+    testIt "renderHTMLQRawMult     " renderHTMLQRawMult
+    testIt "renderHTMLQRaw2        " renderHTMLQRaw2
+    testIt "renderHTMLQRaw3        " renderHTMLQRaw3
     testIt "renderHTMLQRaw4        " renderHTMLQRaw4
-    ]
+    it     "singleAlways           " singleAlways
+    it     "doubleAlways           " doubleAlways
+    it     "singleAvoidEscaping    " singleAvoidEscaping
+    it     "doubleAvoidEscaping    " doubleAvoidEscaping
 
 renderHTMLVoid :: Bool
 renderHTMLVoid =
     toByteString (render (HtmlDocument UTF8 Nothing [
         Element "img" [("src", "foo")] []
         ]))
-    == "<img src=\'foo\' />"
+    == "<img src='foo' />"
 
 renderHTMLVoid2 :: Bool
 renderHTMLVoid2 = isBottom $
@@ -806,7 +782,7 @@ renderHTMLRaw =
             TextNode "<testing>/&+</foo>"
             ]
         ]))
-    == "<script type=\'text/javascript\'><testing>/&+</foo></script>"
+    == "<script type='text/javascript'><testing>/&+</foo></script>"
 
 renderHTMLRawMult :: Bool
 renderHTMLRawMult =
@@ -816,7 +792,7 @@ renderHTMLRawMult =
             TextNode "bar"
             ]
         ]))
-    == "<script type=\'text/javascript\'>foobar</script>"
+    == "<script type='text/javascript'>foobar</script>"
 
 renderHTMLRaw2 :: Bool
 renderHTMLRaw2 = isBottom $
@@ -855,32 +831,32 @@ renderHTMLEmptyAttr2 =
     toByteString (render (HtmlDocument UTF8 Nothing [
         Element "a" [("href", "")] []
         ]))
-    == "<a href=\"\"></a>"
+    == "<a href=''></a>"
 
 renderHTMLAmpAttr1 :: Bool
 renderHTMLAmpAttr1 =
     toByteString (render (HtmlDocument UTF8 Nothing [
         Element "body" [("foo", "a & b")] [] ]))
-    == "<body foo=\'a & b\'></body>"
+    == "<body foo='a & b'></body>"
 
 renderHTMLAmpAttr2 :: Bool
 renderHTMLAmpAttr2 =
     toByteString (render (HtmlDocument UTF8 Nothing [
         Element "body" [("foo", "a &amp; b")] [] ]))
-    == "<body foo=\'a &amp;amp; b\'></body>"
+    == "<body foo='a &amp;amp; b'></body>"
 
 renderHTMLAmpAttr3 :: Bool
 renderHTMLAmpAttr3 =
     toByteString (render (HtmlDocument UTF8 Nothing [
         Element "body" [("foo", "a &#x65; b")] [] ]))
-    == "<body foo=\'a &amp;#x65; b\'></body>"
+    == "<body foo='a &amp;#x65; b'></body>"
 
 renderHTMLQVoid :: Bool
 renderHTMLQVoid =
     toByteString (render (HtmlDocument UTF8 Nothing [
         Element "foo:img" [("src", "foo")] []
         ]))
-    == "<foo:img src=\'foo\' />"
+    == "<foo:img src='foo' />"
 
 renderHTMLQVoid2 :: Bool
 renderHTMLQVoid2 = isBottom $
@@ -895,7 +871,7 @@ renderHTMLQRaw =
             TextNode "<testing>/&+</foo>"
             ]
         ]))
-    == "<foo:script type=\'text/javascript\'><testing>/&+</foo></foo:script>"
+    == "<foo:script type='text/javascript'><testing>/&+</foo></foo:script>"
 
 renderHTMLQRawMult :: Bool
 renderHTMLQRawMult =
@@ -905,7 +881,7 @@ renderHTMLQRawMult =
             TextNode "bar"
             ]
         ]))
-    == "<foo:script type=\'text/javascript\'>foobar</foo:script>"
+    == "<foo:script type='text/javascript'>foobar</foo:script>"
 
 renderHTMLQRaw2 :: Bool
 renderHTMLQRaw2 = isBottom $
@@ -932,22 +908,61 @@ renderHTMLQRaw4 = isBottom $
             ]
         ]))
 
+renderToByteString :: RenderOptions -> ByteString
+renderToByteString opts = toByteString (renderWithOptions opts document)
+  where
+    attrs :: [(Text, Text)]
+    attrs = [("single", "'"), ("double", "\""), ("both", "'\"")]
+    document :: Document
+    document = HtmlDocument UTF8 Nothing [Element "div" attrs []]
+
+singleAlways :: Assertion
+singleAlways =
+    assertEqual "singleAlways"
+                (renderToByteString (RenderOptions SurroundSingleQuote
+                                                   AttrResolveByEscape
+                                                   Nothing))
+                "<div single='&apos;' double='\"' both='&apos;\"'></div>"
+
+doubleAlways :: Assertion
+doubleAlways =
+    assertEqual "doubleAlways"
+                (renderToByteString (RenderOptions SurroundDoubleQuote
+                                                   AttrResolveByEscape
+                                                   Nothing))
+                "<div single=\"'\" double=\"&quot;\" both=\"'&quot;\"></div>"
+
+singleAvoidEscaping :: Assertion
+singleAvoidEscaping =
+    assertEqual "singleAvoidEscaping"
+                (renderToByteString (RenderOptions SurroundSingleQuote
+                                                   AttrResolveAvoidEscape
+                                                   Nothing))
+                "<div single=\"'\" double='\"' both='&apos;\"'></div>"
+
+doubleAvoidEscaping :: Assertion
+doubleAvoidEscaping =
+    assertEqual "doubleAvoidEscaping"
+                (renderToByteString (RenderOptions SurroundDoubleQuote
+                                                   AttrResolveAvoidEscape
+                                                   Nothing))
+                "<div single=\"'\" double='\"' both=\"'&quot;\"></div>"
+
 
 ------------------------------------------------------------------------------
 -- Tests of rendering from the blaze-html package ----------------------------
 ------------------------------------------------------------------------------
 
-blazeRenderTests :: [Test]
-blazeRenderTests = [
-    testIt   "blazeTestString        " blazeTestString,
-    testIt   "blazeTestText          " blazeTestText,
-    testIt   "blazeTestBS            " blazeTestBS,
-    testIt   "blazeTestPre           " blazeTestPre,
-    testIt   "blazeTestExternal      " blazeTestExternal,
-    testIt   "blazeTestCustom        " blazeTestCustom,
-    testIt   "blazeTestMulti         " blazeTestMulti,
+blazeRenderTests :: Spec
+blazeRenderTests = do
+    testIt   "blazeTestString        " blazeTestString
+    testIt   "blazeTestText          " blazeTestText
+    testIt   "blazeTestBS            " blazeTestBS
+    testIt   "blazeTestPre           " blazeTestPre
+    testIt   "blazeTestExternal      " blazeTestExternal
+    testIt   "blazeTestCustom        " blazeTestCustom
+    testIt   "blazeTestMulti         " blazeTestMulti
     testIt   "blazeTestEmpty         " blazeTestEmpty
-    ]
 
 blazeTestIsString :: (IsString t1, IsString t) =>
      (t -> AttributeValue) -> (t1 -> Html) -> Bool
@@ -997,6 +1012,3 @@ blazeTestEmpty = renderHtmlNodes mempty == []
 
 selectCustom :: Html
 selectCustom = H.select ! H.customAttribute "dojoType" "select" $ (mappend "foo " "bar")
-
-
-
